#ifndef _MUSSA_NWAY_H_
#define _MUSSA_NWAY_H_
//  This file is part of the Mussa source distribution.
//  http://mussa.caltech.edu/
//  Contact author: Tristan  De Buysscher, tristan@caltech.edu

// This program and all associated source code files are Copyright (C) 2005
// the California Institute of Technology, Pasadena, CA, 91125 USA.  It is
// under the GNU Public License; please see the included LICENSE.txt
// file for more information, or contact Tristan directly.


//                        ----------------------------------------
//                         ----------  mussa_nway.hh  -----------
//                        ----------------------------------------
#include <QObject>
#include <QString>

#include <boost/filesystem/path.hpp>

#include <list>
#include <string>
#include <vector>

#include "alg/conserved_path.hpp"
#include "alg/flp.hpp"
#include "alg/sequence.hpp"

class NwayPaths : public QObject
{
  Q_OBJECT

signals:
  //! emit to indicate how much progress we've made
  void progress(const QString& description, int cur, int max);

public:
    typedef size_t size_type;
    typedef std::list<ConservedPath> ConservedPaths;
    
    NwayPaths();
    NwayPaths(const NwayPaths&);

    //! setup an nway comparison, initialize # of species, window size, 
    //! threshold
    void setup(int w, int t);
    void setup_ent(double new_entropy_thres, std::vector<Sequence> some_Seqs);
    //! clear out our path
    void clear();
    //! get the "soft" threshold (between the hard threshold and window size)
    int get_soft_threshold() const;
    //! set the score that a match must exceed inorder to be recorded as a path
    void set_soft_threshold(int soft_thres);
    //! return minimum threshold for this analysis
    int get_threshold() const;
    //! return window size used for this analysis
    int get_window() const;

    void radiate_path_search(std::vector<std::vector<FLPs> > all_comparisons);
    void trans_path_search(std::vector<std::vector<FLPs> > all_comparisons);
    void entropy_path_search(std::vector<std::vector<FLPs> > all_comparisons);
    double path_entropy(std::vector<int> path);

  // old recursive transitive nway ... has issues checking all links?
    void find_paths_r(std::vector<std::vector<FLPs> > all_comparisons);
    void path_search(std::vector<std::vector<FLPs> > all_comparisons, ConservedPath path, size_type depth);

    void simple_refine();
    void save(boost::filesystem::path save_file_path);
    //! load a muway file, \throws mussa_load_error
    void load(boost::filesystem::path load_file_path);
    void add_path(int threshold, std::vector<int>& loaded_path);
    void add_path(ConservedPath loaded_path);
    //! how many sequences are in our comparison
    size_type sequence_count() const;

    void find_paths(std::vector<std::vector<FLPs> > all_comparisons);
    void refine();

    void save_old(boost::filesystem::path save_file_path);
    void print(std::list<std::vector<int> >&);

    // The following iterator functions are mostly for the python interface
    // they'll have problems when being called from within a const object
    ConservedPaths::iterator pbegin() { return pathz.begin() ; }
    ConservedPaths::iterator pend() { return pathz.end() ; }
    size_type path_size() const { return pathz.size(); }
    ConservedPaths::iterator rpbegin() { return refined_pathz.begin() ; }
    ConservedPaths::iterator rpend() { return refined_pathz.end() ; }
    size_type refined_path_size() const { return refined_pathz.size(); }

    size_type size() const;
    // these probably shouldn't be public, but lets start 
    // simple
    ConservedPaths pathz;
    ConservedPaths refined_pathz;

protected:
    int threshold;
    size_type win_size;
    int soft_thres;

    double ent_thres;
    std::vector<char *> c_sequences; //used by entropy_path_search
};
#endif
